@@ -270,8 +270,9 @@ FileAnnotation* Annotate::getFileAnnotation(SCRef sha) {
 
 void Annotate::setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* fa) {
 
-	QString f;
-	SCRef fileSha(git->getFileSha(fileName, sha)); // calls Qt event loop
+	QString fileSha;
+	QByteArray fileData;
+	git->getFile(fileName, sha, NULL, &fileData, &fileSha); // calls Qt event loop
 	if (cancelingAnnotate)
 		return;
 
@@ -281,13 +282,9 @@ void Annotate::setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* f
 		return;
 	}
 	ah[sha].fileSha = fileSha;
-
-	git->getFile(fileName, sha, NULL, &f); // blocking call, calls Qt event loop
-	if (cancelingAnnotate)
-		return;
-
-	int lineNum = f.contains('\n');
-	if (!f.endsWith("\n")) // No newline at end of file
+	QString fileTxt(fileData);
+	int lineNum = fileTxt.contains('\n');
+	if (!fileTxt.endsWith("\n")) // No newline at end of file
 		lineNum++;
 
 	fa->lines.insert(fa->lines.end(), lineNum, "");
@@ -388,8 +385,8 @@ void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
 			// message is locale dependent, so just test the space after '\'
 			if (line[1] == ' ')
 				break;
-			else
-				; // fall through
+
+			// fall through
 		default:
 			++cur;
 			break;
@@ -740,6 +737,12 @@ const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 		if (fa.fileSha == fileSha)
 			return histRevOrder[*shaIdx];
 	}
+	// ok still not found, this could happen if sha is an unapplied
+	// stgit patch. In this case fall back on the first in the list
+	// that is the newest.
+	if (git->getAllRefSha(Git::UN_APPLIED).contains(sha))
+		return histRevOrder.first();
+
 	dbp("ASSERT in getAncestor: ancestor of %1 not found", sha);
 	return "";
 }
