@@ -1,5 +1,5 @@
 /*
-	Description: file names persistant cache
+	Description: file names persistent cache
 
 	Author: Marco Costalba (C) 2005-2006
 
@@ -62,25 +62,46 @@ bool Cache::save(const QString& gitDir, const RevFileMap& rf,
 	buf.reserve(bufSize);
 	QDictIterator<RevFile> it(rf);
 	for ( ; it.current(); ++it) {
-		if (   it.currentKey() == ZERO_SHA
-		    || it.currentKey() == CUSTOM_SHA
-		    || it.currentKey().startsWith("A")) // ALL_MERGE_FILES + rev sha
+
+		SCRef sha = it.currentKey();
+		if (   sha == ZERO_SHA
+		    || sha == CUSTOM_SHA
+		    || sha.startsWith("A")) // ALL_MERGE_FILES + rev sha
 			continue;
 
-		buf.append(it.currentKey());
+		buf.append(sha);
 	}
 	stream << buf;
 
 	for (it.toFirst(); it.current(); ++it) {
-		if (   it.currentKey() == ZERO_SHA
-		    || it.currentKey() == CUSTOM_SHA
-		    || it.currentKey().startsWith("A")) // ALL_MERGE_FILES + rev sha
+
+		SCRef sha = it.currentKey();
+		if (   sha == ZERO_SHA
+		    || sha == CUSTOM_SHA
+		    || sha.startsWith("A")) // ALL_MERGE_FILES + rev sha
 			continue;
 
-		stream << it.current()->names;
-		stream << it.current()->dirs;
-		stream << it.current()->status;
-		stream << it.current()->mergeParent;
+		const RevFile* rf = *it;
+		stream << rf->names;
+		stream << rf->dirs;
+
+		// skip common case of only modified files
+		bool isEmpty = rf->onlyModified;
+		stream << (Q_UINT32)isEmpty;
+		if (!isEmpty)
+			stream << rf->status;
+
+		// skip common case of just one parent
+		isEmpty = (rf->mergeParent.isEmpty() || rf->mergeParent.last() == 1);
+		stream << (Q_UINT32)isEmpty;
+		if (!isEmpty)
+			stream << rf->mergeParent;
+
+		// skip common case of no rename/copies
+		isEmpty = rf->extStatus.isEmpty();
+		stream << (Q_UINT32)isEmpty;
+		if (!isEmpty)
+			stream << rf->extStatus;
 	}
 	dbs("Compressing data...");
 	f.writeBlock(qCompress(data)); // no need to encode with compressed data
@@ -99,8 +120,7 @@ bool Cache::save(const QString& gitDir, const RevFileMap& rf,
 	return true;
 }
 
-bool Cache::load(const QString& gitDir, RevFileMap& rf,
-                 StrVect& dirs, StrVect& files) {
+bool Cache::load(const QString& gitDir, RevFileMap& rfm, StrVect& dirs, StrVect& files) {
 
 	// check for cache file
 	QString path(gitDir + C_DAT_FILE);
@@ -139,17 +159,32 @@ bool Cache::load(const QString& gitDir, RevFileMap& rf,
 	*stream >> buf;
 
 	uint bufIdx = 0;
+	bool isEmpty;
+	Q_UINT32 tmp;
 	while (!stream->atEnd()) {
 
-		SCRef sha(buf.mid(bufIdx, 40));
-		bufIdx += 40;
+		RevFile* rf = new RevFile();
+		*stream >> rf->names;
+		*stream >> rf->dirs;
 
-		rf.insert(sha, new RevFile());
-		RevFile* p = rf[sha];
-		*stream >> p->names;
-		*stream >> p->dirs;
-		*stream >> p->status;
-		*stream >> p->mergeParent;
+		*stream >> tmp;
+		rf->onlyModified = (bool)tmp;
+		if (!rf->onlyModified)
+			*stream >> rf->status;
+
+		*stream >> tmp;
+		isEmpty = (bool)tmp;
+		if (!isEmpty)
+			*stream >> rf->mergeParent;
+
+		*stream >> tmp;
+		isEmpty = (bool)tmp;
+		if (!isEmpty)
+			*stream >> rf->extStatus;
+
+		SCRef sha(buf.mid(bufIdx, 40));
+		rfm.insert(sha, rf);
+		bufIdx += 40;
 	}
 	f.close();
 	delete stream;
