@@ -233,9 +233,12 @@ namespace QGit {
 	void writeSetting(SCRef key, SCRef value, SCRef group = "");
 
 	// misc helpers
-	bool stripPartialParaghraps(SCRef src, QString* dst, QString* prev);
-	bool writeToFile(SCRef fileName, SCRef data);
+	bool stripPartialParaghraps(const QByteArray& src, QString* dst, QString* prev);
+	bool writeToFile(SCRef fileName, SCRef data, bool setExecutable = false);
+	bool writeToFile(SCRef fileName, const QByteArray& data, bool setExecutable = false);
 	bool readFromFile(SCRef fileName, QString& data);
+	void baAppend(QByteArray** src, const char* ascii, int len);
+	void baAppend(QByteArray& ba, const QByteArray& data);
 
 	// cache file
 	const uint C_MAGIC  = 0xA0B0C0D0;
@@ -264,11 +267,11 @@ public:
 		descRefsMaster = ancRefsMaster = descBrnMaster = -1;
 		*next = indexData();
 	}
-	bool isBoundary() const { return (boundaryOfs == 1); }
+	bool isBoundary() const { return boundary; }
 	uint parentsCount() const { return parentsCnt; }
 	const QString parent(int idx) const;
 	const QStringList parents() const;
-	const QString sha() const { return mid(start + boundaryOfs, 40); }
+	const QString sha() const { return mid(start + startOfs, 40); }
 	const QString author() const { return mid(autStart, autLen); }
 	const QString authorDate() const { return mid(autDateStart, autDateLen); }
 	const QString shortLog() const { return mid(sLogStart, sLogLen); }
@@ -290,7 +293,8 @@ private:
 
 	const QByteArray& ba; // reference here!
 	const uint start;
-	uint parentsCnt, boundaryOfs;
+	bool boundary;
+	uint parentsCnt, startOfs;
 	int autStart, autLen, autDateStart, autDateLen;
 	int sLogStart, sLogLen, lLogStart, lLogLen;
 };
@@ -327,7 +331,7 @@ public:
 		COPIED   = 16,
 		UNKNOWN  = 32,
 		IN_INDEX = 64,
-		ANY      = 128
+		ANY      = 127
 	};
 
 	RevFile() : onlyModified(true) {}
