@@ -52,17 +52,22 @@ bool ConsoleImpl::start(const QString& cmd, const QString& cmdArgs) {
 
 	statusBar()->message("Executing \'" + name + "\' action...");
 
-	QString t(cmd.section('\n', 1, 0xffffffff, QString::SectionIncludeLeadingSep));
-
 	// in case of a multi-sequence, line arguments are bounded to first command only
-	QString txt = cmd.section('\n', 0, 0).append(cmdArgs).append(t);
-	textLabelCmd->setText(txt);
+	QString txt = cmd.section('\n', 0, 0).stripWhiteSpace();
+	QString args = cmdArgs.stripWhiteSpace();
+	QString tail(cmd.section('\n', 1).stripWhiteSpace());
 
-	if (t.stripWhiteSpace().isEmpty())
-		// any one-line command followed by a newline would fail
-		proc = git->runAsync(cmd.stripWhiteSpace(), this);
+	if (!args.isEmpty())
+		txt.append(' ' + args);
+
+	if (!tail.isEmpty()) // any one-line command followed by a newline would fail
+		txt.append('\n' + tail);
+
+	textLabelCmd->setText(txt);
+	if (tail.isEmpty())
+		proc = git->runAsync(txt, this);
 	else
-		proc = git->runAsScript(cmd, this); // wrap in a script
+		proc = git->runAsScript(txt.append('\n'), this); // wrap in a script
 
 	if (proc.isNull())
 		deleteLater();
@@ -72,7 +77,7 @@ bool ConsoleImpl::start(const QString& cmd, const QString& cmdArgs) {
 	return !proc.isNull();
 }
 
-void ConsoleImpl::on_procDataReady(const QString& data) {
+void ConsoleImpl::on_procDataReady(const QByteArray& data) {
 
 	QString newParagraph;
 	if (QGit::stripPartialParaghraps(data, &newParagraph, &inpBuf))
