@@ -25,7 +25,7 @@ signals:
 	void customAction_exited(const QString& name);
 
 public slots:
-	void on_procDataReady(const QString& data);
+	void on_procDataReady(const QByteArray& data);
 	void on_eof();
 
 protected slots:
