@@ -30,7 +30,7 @@ public:
 	bool start(const QStringList& args, const QString& wd);
 
 public slots:
-	void on_procDataReady(const QString&);
+	void on_procDataReady(const QByteArray&);
 	void on_eof();
 
 signals:
@@ -44,7 +44,6 @@ private slots:
 
 private:
 	void parseSingleBuffer(const QByteArray& ba);
-	void baAppend(QByteArray** src, const char* ascii, int len);
 	void addSplittedChunks(const QByteArray* halfChunk);
 	bool doStart(const QStringList& args, const QString& wd);
 	ulong readNewData(bool lastBuffer);
