@@ -20,19 +20,6 @@ void StateInfo::S::clear() {
 	sel = true;
 }
 
-StateInfo::S& StateInfo::S::operator=(const StateInfo::S& st) {
-
-	if (&st != this) {
-		sha   = st.sha;
-		fn    = st.fn;
-		dtSha = st.dtSha;
-		sel   = st.sel;
-		isM   = st.isM;
-		allM  = st.allM;
-	}
-	return *this;
-}
-
 bool StateInfo::S::operator==(const StateInfo::S& st) const {
 
 	if (&st == this)
@@ -106,6 +93,7 @@ bool StateInfo::isChanged(uint what) const {
 Domain::Domain(MainImpl* m, Git* g) : QObject(m), git(g) {
 
 	EM_INIT(exDeleteRequest, "Deleting domain");
+	EM_INIT(exCancelRequest, "Canceling update");
 
 	st.clear();
 	busy = readyToDrag = dragging = dropping = linked = false;
@@ -162,7 +150,7 @@ MainImpl* Domain::m() const {
 	return static_cast<MainImpl*>(parent());
 }
 
-QString Domain::dragHostName() const {
+const QString Domain::dragHostName() const {
 
 	return QString::fromLatin1("@") + m()->curWorkDir();
 }
@@ -251,10 +239,13 @@ void Domain::populateState() {
 
 void Domain::update(bool fromMaster, bool force) {
 
-	if (busy)
-		// quick exit current (obsoleted) update
+	if (busy && st.requestPending()) {
+		// quick exit current (obsoleted) update but only if state
+		// is going to change. Without this check calling update()
+		// many times with the same data nullify the update
+		EM_RAISE(exCancelRequest);
 		emit cancelDomainProcesses();
-
+	}
 	if (busy || dragging)
 		return;
 
@@ -266,6 +257,7 @@ void Domain::update(bool fromMaster, bool force) {
 		return;
 	}
 	try {
+		EM_REGISTER_Q(exCancelRequest); // quiet, no messages when thrown
 		setThrowOnDelete(true);
 		git->setThrowOnStop(true);
 		git->setCurContext(this);
@@ -288,15 +280,28 @@ void Domain::update(bool fromMaster, bool force) {
 		git->setCurContext(NULL);
 		git->setThrowOnStop(false);
 		setThrowOnDelete(false);
+		EM_REMOVE(exCancelRequest);
 
-	} catch(int i) {
+	} catch (int i) {
 
-		st.rollBack();
+		/*
+		   If we have a cancel request because of a new update is in queue we
+		   cannot roolback current state to avoid new update is filtered out
+		   in case rolled back sha and new sha are the same.
+		   This could happen with arrow navigation:
+
+		   sha -> go UP (new sha) -> go DOWN (cancel) -> rollback to sha
+
+		   And pending request 'sha' is filtered out leaving us in an
+		   inconsistent state.
+		*/
+		st.commit();
 		st.setLock(false);
 		busy = false;
 		git->setCurContext(NULL);
 		git->setThrowOnStop(false);
 		setThrowOnDelete(false);
+		EM_REMOVE(exCancelRequest);
 
 		if (QApplication::overrideCursor())
 			QApplication::restoreOverrideCursor();
@@ -310,10 +315,15 @@ void Domain::update(bool fromMaster, bool force) {
 			EM_THROW_PENDING;
 			return;
 		}
-		const QString info("Exception \'" + EM_DESC(i) + "\' "
-		                   "not handled in init...re-throw");
-		dbs(info);
-		throw;
+		if (i == exCancelRequest)
+			EM_THROW_PENDING;
+			// do not return
+		else {
+			const QString info("Exception \'" + EM_DESC(i) + "\' "
+			                   "not handled in init...re-throw");
+			dbs(info);
+			throw;
+		}
 	}
 	bool nextRequestPending = flushQueue();
 
