@@ -28,7 +28,7 @@ public:
 	FileContent(Domain* parent, Git* git, QTextEdit* f);
 	~FileContent();
 	void update(bool force = false);
-	bool annotateAvailable() { return curAnn != NULL; };
+	bool annotateAvailable() { return curAnn != NULL; }
 	void clear();
 	void copySelection();
 	void goToAnnotation(int id);
@@ -47,7 +47,7 @@ signals:
 public slots:
 	void on_doubleClicked(int, int);
 	void on_annotateReady(Annotate*, const QString&, bool, const QString&);
-	void on_procDataReady(const QString&);
+	void on_procDataReady(const QByteArray&);
 	void on_eof(bool emitSignal = true);
 
 private:
@@ -60,7 +60,7 @@ private:
 	uint annotateLength(const FileAnnotation* curAnn);
 	void saveScreenState();
 	void restoreScreenState();
-	uint processData(const QString& fileChunk);
+	uint processData(const QByteArray& fileChunk);
 
 	Domain* d;
 	Git* git;
@@ -71,7 +71,7 @@ private:
 	QGuardedPtr<Annotate> annotateObj;
 	QGuardedPtr<MyProcess> proc;
 	const FileAnnotation* curAnn;
-	QString fileRowData;
+	QByteArray fileRowData;
 	QString fileProcessedData;
 	QString halfLine;
 	uint curLine;
