@@ -63,12 +63,12 @@ void ListBoxFiles::update(const RevFile* files, bool newFiles) {
 		lb->setPaletteBackgroundColor(QGit::LIGHT_BLUE);
 
 	if (newFiles)
-		insertFiles(files);
+		insertFiles(files); // currentItem is NULL after this call
 
 	QString fileName(lb->currentText());
 	git->removeExtraFileInfo(&fileName); // could be a renamed/copied file
 
-	if (fileName && (fileName == st->fileName())) {
+	if (!fileName.isEmpty() && (fileName == st->fileName())) {
 		lb->setSelected(lb->currentItem(), st->selectItem()); // just a refresh
 		return;
 	}
