@@ -29,7 +29,7 @@ public:
 	ListBoxFiles(Domain* mi, Git* g, QListBox* l);
 
 	void clear();
-	void update(const RevFile* files);
+	void update(const RevFile* files, bool newFiles);
 
 signals:
 	void contextMenu(const QString&, int);
