@@ -152,15 +152,13 @@ bool FileView::doUpdate(bool force) {
 
 		clear(false);
 		m()->tabWdg->changeTab(fileTab, st.fileName());
-
-		QApplication::restoreOverrideCursor();
-		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-
-		m()->statusBar()->message("Retrieving history of '" +
-		                          st.fileName() + "'...");
 		fh->clear(st.fileName());
-		git->startFileHistory(fh);
-
+		if (git->startFileHistory(fh)) {
+			QApplication::restoreOverrideCursor();
+			QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+			m()->statusBar()->message("Retrieving history of '" +
+			                           st.fileName() + "'...");
+		}
 	} else if (histListView->update() || st.sha().isEmpty()) {
 
 		updateSpinBoxValue();
@@ -232,7 +230,7 @@ void FileView::on_toolButtonPin_toggled(bool b) {
 	fileTab->spinBoxRevision->setDisabled(b);
 
 	if (!b) {
-		updateSpinBoxValue();
+		updateSpinBoxValue(); // UPDATE() call is filtered in this case
 		textEditFile->update(true);
 	}
 }
@@ -337,8 +335,17 @@ void FileView::on_fileAvailable(bool b) {
 
 void FileView::on_revIdSelected(int id) {
 
-	if (id != 0 && fileTab->spinBoxRevision->isEnabled())
+	if (id == 0)
+		return;
+
+	if (fileTab->spinBoxRevision->isEnabled())
 		fileTab->spinBoxRevision->setValue(id);
+	else {
+		QListView* hv = fileTab->histListView;
+		QString row = QString::number(id);
+		QListViewItem* item = hv->findItem(row, QGit::ANN_ID_COL, Qt::Contains);
+		hv->setCurrentItem(item);
+	}
 }
 
 // ******************************* data events ****************************
