@@ -7,8 +7,6 @@
 
 */
 #include <stdlib.h>    // used by getenv()
-#include <sys/types.h> // used by chmod()
-#include <sys/stat.h>  // used by chmod()
 #include <qapplication.h>
 #include <qdatetime.h>
 #include <qtextcodec.h>
@@ -120,6 +118,20 @@ void Git::userInfo(SList info) {
 	errorReportingEnabled = true;
 }
 
+bool Git::isBinaryFile(SCRef file) {
+
+	static const char* binaryFileExtensions[] = {"bmp", "gif", "jpeg", "jpg",
+	                   "png", "svg", "tiff", "pcx", "xcf", "xpm",
+	                   "bz", "bz2", "rar", "tar", "z", "gz", "tgz", "zip", 0};
+
+	const QString ext(file.section('.', -1).lower());
+	int i = 0;
+	while (binaryFileExtensions[i] != 0)
+		if (ext == binaryFileExtensions[i++])
+			return true;
+	return false;
+}
+
 void Git::setThrowOnStop(bool b) {
 
 	if (b)
@@ -454,6 +466,20 @@ const Rev* Git::revLookup(SCRef sha, const FileHistory* fh) const {
 
 bool Git::run(SCRef runCmd, QString* runOutput, QObject* receiver, SCRef buf) {
 
+	if (!runOutput)
+		return run(NULL, runCmd, receiver, buf);
+
+	*runOutput = "";
+	QByteArray ba;
+	bool ret = run(&ba, runCmd, receiver, buf);
+	if (!ba.isEmpty())
+		runOutput->append(ba);
+
+	return ret;
+}
+
+bool Git::run(QByteArray* runOutput, SCRef runCmd, QObject* receiver, SCRef buf) {
+
 	MyProcess p(parent(), this, workDir, errorReportingEnabled);
 	return p.runSync(runCmd, runOutput, receiver, buf);
 }
@@ -471,10 +497,9 @@ MyProcess* Git::runAsync(SCRef runCmd, QObject* receiver, SCRef buf) {
 MyProcess* Git::runAsScript(SCRef runCmd, QObject* receiver, SCRef buf) {
 
 	const QString scriptFile(workDir + "/qgit_script.sh");
-	if (!writeToFile(scriptFile, runCmd))
+	if (!writeToFile(scriptFile, runCmd, true))
 		return NULL;
 
-	chmod(scriptFile, 0755);
 	MyProcess* p = runAsync(scriptFile, receiver, buf);
 	if (p)
 		connect(p, SIGNAL(eof()), this, SLOT(on_runAsScript_eof()));
@@ -584,7 +609,13 @@ MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool comb
 
 const QString Git::getFileSha(SCRef file, SCRef revSha) {
 
-	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha); // FIXME ZERO_SHA != HEAD
+	if (revSha == ZERO_SHA) {
+		QStringList files, dummy;
+		getWorkDirFiles(files, dummy, RevFile::ANY);
+		if (files.contains(file))
+			return ZERO_SHA; // it is unknown to git
+	}
+	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha);
 	QString runCmd("git ls-tree -r " + sha + " " + quote(file)), runOutput;
 	if (!run(runCmd, &runOutput))
 		return "";
@@ -592,26 +623,40 @@ const QString Git::getFileSha(SCRef file, SCRef revSha) {
 	return runOutput.mid(12, 40); // could be empty, deleted file case
 }
 
-MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* result) {
+MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver,
+                        QByteArray* result, QString* fSha) {
 
 	QString runCmd;
-	if (revSha == ZERO_SHA)
+	/*
+	  symlinks in git are one line files with just the name of the target,
+	  not the target content. Instead 'cat' command resolves symlinks and
+	  returns target content. So we use 'cat' only if the file is modified
+	  in working dir, to let annotation work for changed files, otherwise
+	  we go with a safe 'git cat-file blob <file sha of HEAD>' instead.
+	  NOTE: This fails if the modified file is a new symlink, converted
+	  from an old plain file. In this case annotation will fail until
+	  change is committed.
+	*/
+	const QString fileSha(getFileSha(file, revSha));
+	if (fileSha == ZERO_SHA)
 		runCmd = "cat " + quote(file);
 	else {
-		SCRef fileSha(getFileSha(file, revSha));
-		if (!fileSha.isEmpty())
-			runCmd = "git cat-file blob " + fileSha;
-		else
+		if (fileSha.isEmpty()) // deleted
 			runCmd = "git diff-tree HEAD HEAD"; // fake an empty file reading
+		else
+			runCmd = "git cat-file blob " + fileSha;
 	}
+	if (fSha)
+		*fSha = fileSha;
+
 	if (!receiver) {
-		run(runCmd, result);
+		run(result, runCmd);
 		return NULL; // in case of sync call we ignore run() return value
 	}
 	return runAsync(runCmd, receiver);
 }
 
-MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QString* result){
+MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QString* result) {
 
 	if (!isTextHighlighter()) {
 		dbs("ASSERT in getHighlightedFile: highlighter not found");
@@ -645,9 +690,12 @@ void Git::on_getHighlightedFile_eof() {
 
 bool Git::saveFile(SCRef file, SCRef sha, SCRef path) {
 
-	QString fileData;
+	QByteArray fileData;
 	getFile(file, sha, NULL, &fileData); // sync call
-	return writeToFile(path, fileData);
+	if (isBinaryFile(file))
+		return writeToFile(path, fileData);
+
+	return writeToFile(path, QString(fileData));
 }
 
 bool Git::getTree(SCRef treeSha, SList names, SList shas,
@@ -782,8 +830,14 @@ const QStringList Git::getDescendantBranches(SCRef sha) {
 		SCRef sha = revData.revOrder[nr[i]];
 		SCRef cap = " (" + sha + ") ";
 		RefMap::const_iterator it(refsShaMap.find(sha));
-		if (it != refsShaMap.constEnd())
+		if (it == refsShaMap.constEnd())
+			continue;
+
+		if (!(*it).branches.empty())
 			tl.append((*it).branches.join(cap).append(cap));
+
+		if (!(*it).remoteBranches.empty())
+			tl.append((*it).remoteBranches.join(cap).append(cap));
 	}
 	return tl;
 }
@@ -852,7 +906,7 @@ const QString Git::colorMatch(SCRef txt, QRegExp& regExp) {
 	return text;
 }
 
-const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
+const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE, bool showHeader) {
 
 	if (sha.isEmpty())
 		return "";
@@ -865,29 +919,35 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 	if (c->isDiffCache)
 		text = c->longLog();
 	else {
-		text = QString("Author: " + c->author() + "\nDate:   ");
-		text.append(getLocalDate(c->authorDate()));
-		if (!c->isUnApplied && !c->isApplied) {
-			text.append("\nParent: ").append(c->parents().join("\nParent: "));
+		if (showHeader) {
+			text = QString("Author: " + c->author() + "\nDate:   ");
+			text.append(getLocalDate(c->authorDate()));
+			if (!c->isUnApplied && !c->isApplied) {
+				text.append("\nParent: ").append(c->parents().join("\nParent: "));
 
-			QStringList sl = getChilds(sha);
-			if (!sl.isEmpty())
-				text.append("\nChild: ").append(sl.join("\nChild: "));
+				QStringList sl = getChilds(sha);
+				if (!sl.isEmpty())
+					text.append("\nChild: ").append(sl.join("\nChild: "));
 
-			sl = getDescendantBranches(sha);
-			if (!sl.empty())
-				text.append("\nBranch: ").append(sl.join("\nBranch: "));
+				sl = getDescendantBranches(sha);
+				if (!sl.empty())
+					text.append("\nBranch: ").append(sl.join("\nBranch: "));
 
-			sl = getNearTags(!optGoDown, sha);
-			if (!sl.isEmpty())
-				text.append("\nFollows: ").append(sl.join(", "));
+				sl = getNearTags(!optGoDown, sha);
+				if (!sl.isEmpty())
+					text.append("\nFollows: ").append(sl.join(", "));
 
-			sl = getNearTags(optGoDown, sha);
-			if (!sl.isEmpty())
-				text.append("\nPrecedes: ").append(sl.join(", "));
+				sl = getNearTags(optGoDown, sha);
+				if (!sl.isEmpty())
+					text.append("\nPrecedes: ").append(sl.join(", "));
+			}
+			text.append("\n\n    " + colorMatch(c->shortLog(), shortLogRE) + "\n");
 		}
-		text.append("\n\n    " + colorMatch(c->shortLog(), shortLogRE) +
-		            '\n' + colorMatch(c->longLog(), longLogRE));
+		const QString log(colorMatch(c->longLog(), longLogRE));
+		if (!showHeader && log.stripWhiteSpace().isEmpty())
+			text.append(colorMatch(c->shortLog(), shortLogRE));
+		else
+			text.append(log);
 	}
 	// workaround a Qt issue
 	QTextCodec* tc = QTextCodec::codecForCStrings();
@@ -997,9 +1057,9 @@ const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef pa
 	return insertNewFiles(sha, runOutput);
 }
 
-void Git::startFileHistory(FileHistory* fh) {
+bool Git::startFileHistory(FileHistory* fh) {
 
-	startRevList(quote(fh->fileName), fh);
+	return startRevList(fh->fileName, fh);
 }
 
 void Git::getFileFilter(SCRef path, QMap<QString, bool>& shaMap) {
@@ -1053,13 +1113,16 @@ bool Git::resetCommits(int parentDepth) {
 	return run(runCmd);
 }
 
-bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool isDragDrop) {
+bool Git::applyPatchFile(SCRef patchPath, bool fold, bool isDragDrop) {
 
-	if (commit && isStGIT) {
-		if (fold)
-			return run("stg fold " + quote(patchPath));
-
-		return run("stg import --mail " + quote(patchPath));
+	if (isStGIT) {
+		if (fold) {
+			bool ok = run("stg fold " + quote(patchPath)); // merge in working dir
+			if (ok)
+				ok = run("stg refresh"); // update top patch
+			return ok;
+		} else
+			return run("stg import --mail " + quote(patchPath));
 	}
 	QString runCmd("git am --utf8 --3way ");
 	if (isDragDrop)
