@@ -45,7 +45,8 @@ public:
 		optOnlyLoaded,
 		optDragDrop,
 		optFold,
-		optOnlyInIndex
+		optOnlyInIndex,
+		optCreate
 	};
 
 	enum RefType {
@@ -70,27 +71,28 @@ public:
 	Annotate* startAnnotate(FileHistory* fh, QObject* guiObj);
 	const FileAnnotation* lookupAnnotation(Annotate* ann, SCRef fileName, SCRef sha);
 	void cancelAnnotate(Annotate* ann);
-	void startFileHistory(FileHistory* fh);
+	bool startFileHistory(FileHistory* fh);
 	void cancelDataLoading(const FileHistory* fh);
 	void cancelProcess(MyProcess* p);
 	bool isCommittingMerge() const { return isMergeHead; }
 	bool isStGITStack() const { return isStGIT; }
 	bool isPatchName(SCRef nm);
 	bool isSameFiles(SCRef tree1Sha, SCRef tree2Sha);
+	bool isBinaryFile(SCRef file);
 	bool isNothingToCommit();
 	bool isUnknownFiles() const { return (_wd.otherFiles.count() > 0); }
 	bool isTextHighlighter() const { return isTextHighlighterFound; }
 	bool isMainHistory(const FileHistory* fh) { return (fh == &revData); }
 	MyProcess* getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool combined);
-	MyProcess* getFile(SCRef file, SCRef revSha, QObject* receiver, QString* runOutput);
-	MyProcess* getHighlightedFile(SCRef file, SCRef revSha, QObject* rcv, QString* ro);
+	MyProcess* getFile(SCRef file, SCRef revSha, QObject* rcv, QByteArray* ro = NULL, QString* fSha = NULL);
+	MyProcess* getHighlightedFile(SCRef file, SCRef revSha, QObject* rcv, QString* ro = NULL);
 	bool saveFile(SCRef fileName, SCRef sha, SCRef path);
 	void getFileFilter(SCRef path, QMap<QString, bool>& shaMap);
 	bool getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap);
 	const RevFile* getFiles(SCRef sha, SCRef sha2 = "", bool all = false, SCRef path = "");
 	bool getTree(SCRef ts, SList nm, SList sha, SList type, bool wd, SCRef treePath);
 	static const QString getLocalDate(SCRef gitDate);
-	const QString getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE);
+	const QString getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE, bool showHeader);
 	const QString getDefCommitMsg();
 	const QString getLaneParent(SCRef fromSHA, uint laneNum);
 	const QStringList getChilds(SCRef parent);
@@ -112,7 +114,7 @@ public:
 	bool commitFiles(SCList files, SCRef msg);
 	bool makeTag(SCRef sha, SCRef tag, SCRef msg);
 	bool deleteTag(SCRef sha);
-	bool applyPatchFile(SCRef patchPath, bool commit, bool fold, bool sign);
+	bool applyPatchFile(SCRef patchPath, bool fold, bool sign);
 	bool resetCommits(int parentDepth);
 	bool stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold);
 	bool stgPush(SCRef sha);
@@ -137,7 +139,7 @@ signals:
 	void annotateReady(Annotate*, const QString&, bool, const QString&);
 
 public slots:
-	void on_procDataReady(const QString&);
+	void on_procDataReady(const QByteArray&);
 	void on_eof() { filesLoadingPending = filesLoadingCurSha = ""; }
 
 private slots:
@@ -177,19 +179,18 @@ private:
 	WorkingDirInfo _wd;
 
 	bool run(SCRef cmd, QString* out = NULL, QObject* rcv = NULL, SCRef buf = "");
+	bool run(QByteArray* runOutput, SCRef cmd, QObject* rcv = NULL, SCRef buf = "");
 	MyProcess* runAsync(SCRef cmd, QObject* rcv, SCRef buf = "");
 	MyProcess* runAsScript(SCRef cmd, QObject* rcv = NULL, SCRef buf = "");
 	bool allProcessDeleted();
-	const QString getArgs(bool askForRange, bool* quit);
+	const QStringList getArgs(bool askForRange, bool* quit);
 	bool getRefs();
-	bool getStGITPatches();
-	bool addStGitPatch(SCRef pn, SCList files, SCList filesSHA, bool applied);
-	static void dirWalker(SCRef dirPath, SList files, SList filesSHA, SCRef nameFilter = "");
+	void parseStGitPatches(SCList patchNames, SCList patchShas);
 	void clearRevs();
 	void clearFileNames();
-	bool startRevList(SCRef args, FileHistory* fh);
+	bool startRevList(SCList args, FileHistory* fh);
 	bool startUnappliedList();
-	bool startParseProc(SCRef initCmd, FileHistory* fh);
+	bool startParseProc(SCList initCmd, FileHistory* fh);
 	int addChunk(FileHistory* fh, const QByteArray& ba, int ofs);
 	void parseDiffFormat(RevFile& rf, SCRef buf);
 	void parseDiffFormatLine(RevFile& rf, SCRef line, int parNum);
