@@ -26,7 +26,7 @@
 
 using namespace QGit;
 
-const QString Git::getArgs(bool askForRange, bool* quit) {
+const QStringList Git::getArgs(bool askForRange, bool* quit) {
 
 	static bool startup = true; // it's OK to be unique among qgit windows
 	if (startup) {
@@ -49,15 +49,10 @@ const QString Git::getArgs(bool askForRange, bool* quit) {
 		                    getAllRefNames(TAG, !optOnlyLoaded), quit, this);
 		rs.exec(); // modal execution
 		if (*quit)
-			return "";
+			return QStringList();
 	}
 	startup = false;
-	QString runOutput;
-	if (!run("git rev-parse --default HEAD " + curRange, &runOutput)) {
-		*quit = true;
-		return "";
-	}
-	return runOutput.replace('\n', " ");
+	return MyProcess::splitArgList(curRange);
 }
 
 const QString Git::getBaseDir(bool* changed, SCRef wd, bool* ok, QString* gd) {
@@ -101,10 +96,12 @@ bool Git::getRefs() {
 
 	// check for a StGIT stack
 	QDir d(gitDir);
+	QString stgCurBranch;
 	if (d.exists("patches")) { // early skip
 		errorReportingEnabled = false;
-		isStGIT = run("stg unapplied"); // slow command, check for stg bin
+		isStGIT = run("stg branch", &stgCurBranch); // slow command
 		errorReportingEnabled = true;
+		stgCurBranch = stgCurBranch.stripWhiteSpace();
 	} else
 		isStGIT = false;
 
@@ -128,6 +125,7 @@ bool Git::getRefs() {
 
 	refsShaMap.clear();
 	QString prevRefSha;
+	QStringList patchNames, patchShas;
 	const QStringList rLst(QStringList::split('\n', runOutput));
 	FOREACH_SL (it, rLst) {
 
@@ -135,13 +133,20 @@ bool Git::getRefs() {
 		SCRef refName = (*it).mid(41);
 
 		if (refName.startsWith("refs/patches/")) {
+
+			// save StGIT patch sha, to be used later
+			SCRef patchesDir("refs/patches/" + stgCurBranch + "/");
+			if (refName.startsWith(patchesDir)) {
+				patchNames.append(refName.mid(patchesDir.length()));
+				patchShas.append(revSha);
+			}
 			// StGIT patches should not be added to refs,
 			// but an applied StGIT patch could be also an head or
 			// a tag in this case will be added in another loop cycle
 			continue;
 		}
-		// one rev could have many tags, called with create flag
-		Reference* cur = lookupReference(revSha, true);
+		// one rev could have many tags
+		Reference* cur = lookupReference(revSha, optCreate);
 
 		if (refName.startsWith("refs/tags/")) {
 
@@ -184,97 +189,41 @@ bool Git::getRefs() {
 		}
 		prevRefSha = revSha;
 	}
+	if (isStGIT && !patchNames.isEmpty())
+		parseStGitPatches(patchNames, patchShas);
+
 	return !refsShaMap.empty();
 }
 
-void Git::dirWalker(SCRef dirPath, SList files, SList filesSHA, SCRef nameFilter) {
-// search through dirPath recursively fetching any possible
-// file whom first 40 chars of content are a possible sha
+void Git::parseStGitPatches(SCList patchNames, SCList patchShas) {
 
-	QFileInfo* fi;
-	QDir d(dirPath, "", QDir::Name, QDir::Dirs);
-	QFileInfoListIterator it(*d.entryInfoList());
-	++it; ++it; // first two entries are . and ..
-	while ((fi = it.current()) != 0) {
-		dirWalker(fi->absFilePath(), files, filesSHA, nameFilter);
-		++it;
-	}
-	QString sha;
-	QDir f(dirPath, nameFilter, QDir::Name, QDir::Files);
-	it = *f.entryInfoList();
-	while ((fi = it.current()) != 0) {
-
-		readFromFile(fi->absFilePath(), sha);
-		// we accept also files with a sha + other stuff
-		sha = sha.stripWhiteSpace().append('\n').section('\n', 0, 0);
-		if (sha.length() == 40) {
-			files.append(fi->absFilePath());
-			filesSHA.append(sha);
-		}
-		++it;
-	}
-}
-
-bool Git::addStGitPatch(SCRef patchName, SCList files, SCList filesSHA, bool applied) {
-
-	const QStringList fl(files.grep("/" + patchName + "/top"));
-	if (fl.count() != 1) {
-		qDebug("ASSERT: found %i patches instead of 1 in %s",
-		       (int)fl.count(), patchName.latin1());
-		return false;
-	}
-	int pos = files.findIndex(fl.first());
-	Reference* cur = lookupReference(filesSHA[pos], true);
-	cur->stgitPatch = patchName;
-	cur->type |= (applied ? APPLIED : UN_APPLIED);
-	return true;
-}
-
-bool Git::getStGITPatches() {
+	patchesStillToFind = 0;
 
 	// get patch names and status of current branch
 	QString runOutput;
 	if (!run("stg series", &runOutput))
-		return false;
+		return;
 
-	if (runOutput.isEmpty())
-		return false; // false is used by caller as early exit
-
-	// get current branch
-	QString branch;
-	if (!run("stg branch", &branch))
-		return false;
-
-	branch = branch.stripWhiteSpace();
-
-	QStringList uNames, aNames;
 	const QStringList pl(QStringList::split('\n', runOutput));
 	FOREACH_SL (it, pl) {
 
-		SCRef st = (*it).left(1);
-		SCRef pn = (*it).mid(2);
-		if (st == "+" || st == ">")
-			aNames.append(pn);
-		else
-			uNames.append(pn);
+		SCRef status = (*it).left(1);
+		SCRef patchName = (*it).mid(2);
+
+		bool applied = (status == "+" || status == ">");
+		int pos = patchNames.findIndex(patchName);
+		if (pos == -1) {
+			dbp("ASSERT in Git::parseStGitPatches(), patch %1 "
+			    "not found in references list.", patchName);
+			continue;
+		}
+		Reference* cur = lookupReference(patchShas[pos], optCreate);
+		cur->stgitPatch = patchName;
+		cur->type |= (applied ? APPLIED : UN_APPLIED);
+
+		if (applied)
+			patchesStillToFind++;
 	}
-	// get all sha's in "top" files under /<gitDir>/patches/<current branch>
-	QDir d(gitDir + "/patches/" + branch);
-	QStringList files, filesSHA;
-	dirWalker(d.absPath(), files, filesSHA, "top");
-
-	// now match names and SHA's for unapplied
-	FOREACH_SL (it, uNames)
-		if (!addStGitPatch(*it, files, filesSHA, false))
-			return false;
-
-	// the same for applied, in this case the order is not important
-	FOREACH_SL (it, aNames)
-		if (!addStGitPatch(*it, files, filesSHA, true))
-			return false;
-
-	patchesStillToFind = aNames.count();
-	return true;
 }
 
 const QStringList Git::getOthersFiles() {
@@ -361,7 +310,7 @@ void Git::getDiffIndex() {
 
 	// then mockup the corresponding Rev
 	QString parent;
-	if (!run("git rev-parse --default HEAD", &parent))
+	if (!run("git rev-parse HEAD", &parent))
 		return;
 
 	parent = parent.section('\n', 0, 0);
@@ -380,7 +329,7 @@ void Git::parseDiffFormatLine(RevFile& rf, SCRef line, int parNum) {
 
 		// TODO rename/copy is not supported for combined merges
 		appendFileName(rf, line.section('\t', -1));
-		setStatus(rf, line.section(' ', 6, 6).left(1));
+		setStatus(rf, line.section('\t', 0, 0).section(' ', -1, -1).left(1));
 		rf.mergeParent.append(parNum);
 	} else { // faster parsing in normal case
 
@@ -399,6 +348,7 @@ void Git::setStatus(RevFile& rf, SCRef rowSt) {
 	char status = rowSt.at(0).latin1();
 	switch (status) {
 	case 'M':
+	case 'T':
 		rf.status.append(RevFile::MODIFIED);
 		break;
 	case 'D':
@@ -414,9 +364,9 @@ void Git::setStatus(RevFile& rf, SCRef rowSt) {
 		rf.onlyModified = false;
 		break;
 	default:
-		dbp("ASSERT in Git::setStatus, unknown status %1", rowSt);
-		rf.status.append(RevFile::UNKNOWN);
-		rf.onlyModified = false;
+		dbp("ASSERT in Git::setStatus, unknown status <%1>. "
+		    "'MODIFIED' will be used instead.", rowSt);
+		rf.status.append(RevFile::MODIFIED);
 		break;
 	}
 }
@@ -477,7 +427,7 @@ void Git::parseDiffFormat(RevFile& rf, SCRef buf) {
 	}
 }
 
-bool Git::startParseProc(SCRef initCmd, FileHistory* fh) {
+bool Git::startParseProc(SCList initCmd, FileHistory* fh) {
 
 	DataLoader* dl = new DataLoader(this, fh); // auto-deleted when done
 
@@ -492,38 +442,43 @@ bool Git::startParseProc(SCRef initCmd, FileHistory* fh) {
 	        SLOT(on_loaded(const FileHistory*, ulong, int,
 	        bool, const QString&, const QString&)));
 
-	QStringList args;
-	MyProcess::parseArgs(initCmd, args);
-	return dl->start(args, workDir);
+	return dl->start(initCmd, workDir);
 }
 
-bool Git::startRevList(SCRef args, FileHistory* fh) {
+bool Git::startRevList(SCList args, FileHistory* fh) {
 
-	QString initCmd("git rev-list --header --boundary --parents ");
+	const QString baseCmd("git rev-list --header --parents --boundary --default HEAD");
+	QStringList initCmd(QStringList::split(' ', baseCmd));
 	if (!isMainHistory(fh)) {
 		// fetch history from all branches so any revision in
 		// main view that changes the file is always found
-		SCRef allBranches = getAllRefSha(BRANCH).join(" ");
-		initCmd.append("--remove-empty " + allBranches + " -- ");
+		// NOTE: we don't use '--remove-empty' option because
+		// in case a file is deleted and then a new file with
+		// the same name is created again in the same directory
+		// then, with this option, file history is truncated to
+		// the file deletion revision.
+		initCmd += getAllRefSha(BRANCH | RMT_BRANCH);
+		initCmd += "--";
 	} else
-		initCmd.append("--topo-order ");
+		initCmd += "--topo-order";
 
 	return startParseProc(initCmd + args, fh);
 }
 
 bool Git::startUnappliedList() {
 
+	// TODO truncate sha to avoid overflow in 'git rev-list'
+	// command line arguments
+	const QStringList unAppliedSha(getAllRefSha(UN_APPLIED));
+	if (unAppliedSha.isEmpty())
+		return false;
+
 	// WARNING: with this command git rev-list could send spurious
 	// revs so we need some filter out logic during loading
-	QString initCmd("git rev-list --header --parents ");
-	initCmd.append(getAllRefSha(UN_APPLIED).join(" "));
-	initCmd.append(QString::fromLatin1(" ^HEAD"));
-	loadingUnAppliedPatches = true;
-	bool started = startParseProc(initCmd, &revData);
-	if (!started)
-		loadingUnAppliedPatches = false;
-
-	return started;
+	QStringList initCmd(QStringList::split(' ', "git rev-list --header --parents"));
+	initCmd += unAppliedSha;
+	initCmd += QString::fromLatin1("^HEAD");
+	return startParseProc(initCmd, &revData);
 }
 
 bool Git::stop(bool saveCache) {
@@ -593,7 +548,7 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 			return false;
 		}
 		const QString msg1("Path is '" + workDir + "'    Loading ");
-		QString args;
+		QStringList args;
 		if (!filteredLoading) {
 
 			// update text codec according to repo settings
@@ -612,13 +567,12 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 				setThrowOnStop(false);
 				return false;
 			}
-
 			// load StGit unapplied patches, must be after getRefs()
 			if (isStGIT) {
+				loadingUnAppliedPatches = startUnappliedList();
+				if (loadingUnAppliedPatches) {
 
-				POST_MSG(msg1 + "StGIT unapplied patches...");
-				if (getStGITPatches() && startUnappliedList()) {
-
+					POST_MSG(msg1 + "StGIT unapplied patches...");
 					while (loadingUnAppliedPatches) {
 						// WARNING we are in setRepository() now
 						usleep(20000);
@@ -627,7 +581,6 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 					revData.lns->clear(); // again to reset lanes
 				}
 			}
-
 			// load working dir files
 			if (testFlag(DIFF_INDEX_F)) {
 				POST_MSG(msg1 + "working directory changed files...");
@@ -635,9 +588,9 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 			}
 
 		} else { // filteredLoading
-			SCRef allBranches = getAllRefSha(BRANCH).join(" ");
-			args.append(allBranches + " -- ");
-			args.append(filterList->join(" "));
+			args += getAllRefSha(BRANCH | RMT_BRANCH);
+			args += "--";
+			args += *filterList;
 		}
 		POST_MSG(msg1 + "revisions...");
 		if (!startRevList(args, &revData))
@@ -772,7 +725,7 @@ int Git::addChunk(FileHistory* fh, const QByteArray& ba, int start) {
 		// out unknown revs until no more StGIT patches are waited and
 		// firstNonStGitPatch is reached
 		if (!(firstNonStGitPatch.isEmpty() && patchesStillToFind == 0) &&
-		    !loadingUnAppliedPatches) {
+		    !loadingUnAppliedPatches && isMainHistory(fh)) {
 
 			Reference* rf = lookupReference(sha);
 			if (!(rf && (rf->type & APPLIED))) {
@@ -806,18 +759,20 @@ int Git::addChunk(FileHistory* fh, const QByteArray& ba, int start) {
 			c->isUnApplied = true;
 			c->lanes.append(UNAPPLIED);
 
-		} else if (patchesStillToFind > 0) { // try to avoid costly lookup
+		} else if (patchesStillToFind > 0 || !isMainHistory(fh)) { // try to avoid costly lookup
 
 			Reference* rf = lookupReference(sha);
 			if (rf && (rf->type & APPLIED)) {
 
 				Rev* c = const_cast<Rev*>(revLookup(sha, fh));
 				c->isApplied = true;
-				patchesStillToFind--;
-				if (patchesStillToFind == 0)
-					// any rev will be discarded until
-					// firstNonStGitPatch arrives
-					firstNonStGitPatch = c->parent(0);
+				if (isMainHistory(fh)) {
+					patchesStillToFind--;
+					if (patchesStillToFind == 0)
+						// any rev will be discarded until
+						// firstNonStGitPatch arrives
+						firstNonStGitPatch = c->parent(0);
+				}
 			}
 		}
 	}
@@ -911,10 +866,10 @@ void Git::updateLanes(Rev& c, Lanes& lns, SCRef sha) {
 //	qDebug("%s %s",tmp.latin1(), c.sha.latin1());
 }
 
-void Git::on_procDataReady(const QString& fileChunk) {
+void Git::on_procDataReady(const QByteArray& fileChunk) {
 
 	if (filesLoadingPending.isEmpty())
-		filesLoadingPending = fileChunk;
+		filesLoadingPending = QString(fileChunk);
 	else
 		filesLoadingPending.append(fileChunk); // add to previous half lines
 
@@ -968,11 +923,18 @@ void Git::appendFileName(RevFile& rf, SCRef name) {
 		rf.names.append(*it);
 }
 
+static bool vecContains(const QValueVector<int>& vect, int value) {
+
+	for (uint i = 0, end = vect.count(); i < end; i++)
+		if (vect[i] == value)
+			return true;
+	return false;
+}
+
 void Git::updateDescMap(const Rev* r,uint idx, QMap<QPair<uint, uint>, bool>& dm,
                         QMap<uint, QValueVector<int> >& dv) {
 
-	QValueVector<int> descVec(1, idx);
-
+	QValueVector<int> descVec;
 	if (r->descRefsMaster != -1) {
 
 		const Rev* tmp = revLookup(revData.revOrder[r->descRefsMaster]);
@@ -985,22 +947,33 @@ void Git::updateDescMap(const Rev* r,uint idx, QMap<QPair<uint, uint>, bool>& dm
 				return;
 			}
 			const QValueVector<int>& dvv = dv[nr[i]];
+
+			// copy the whole vector instead of each element
+			// in the first iteration of the loop below
+			descVec = dvv; // quick (shared) copy
+
 			for (uint y = 0; y < dvv.count(); y++) {
 
-				QPair<uint, uint> key = qMakePair(idx, (uint)dvv[y]);
-				QPair<uint, uint> keyN = qMakePair((uint)dvv[y], idx);
+				uint v = (uint)dvv[y];
+				QPair<uint, uint> key = qMakePair(idx, v);
+				QPair<uint, uint> keyN = qMakePair(v, idx);
 				dm.insert(key, true);
 				dm.insert(keyN, false);
-				descVec.append(dvv[y]);
+
+				// we don't want duplicated entry, otherwise 'dvv' grows
+				// greatly in repos with many tagged development branches
+				if (i > 0 && !vecContains(descVec, v)) // i > 0 is rare, no
+					descVec.append(v);             // need to optimize
 			}
 		}
 	}
+	descVec.append(idx);
 	dv.insert(idx, descVec);
 }
 
 void Git::mergeBranches(Rev* p, const Rev* r) {
 
-	int r_descBrnMaster = (checkRef(r->sha(), BRANCH) ? r->orderIdx : r->descBrnMaster);
+	int r_descBrnMaster = (checkRef(r->sha(), BRANCH | RMT_BRANCH) ? r->orderIdx : r->descBrnMaster);
 
 	if (p->descBrnMaster == r_descBrnMaster || r_descBrnMaster == -1)
 		return;
@@ -1089,7 +1062,7 @@ void Git::indexTree() {
 	for (uint i = 0, cnt = ro.count(); i < cnt; i++) {
 
 		uint type = checkRef(ro[i]);
-		bool isB = (type & BRANCH);
+		bool isB = (type & (BRANCH | RMT_BRANCH));
 		bool isT = (type & TAG);
 
 		const Rev* r = revLookup(ro[i]);
@@ -1160,7 +1133,7 @@ const QString Rev::mid(int start, int len) const {
 
 const QString Rev::parent(int idx) const {
 
-	return mid(start + boundaryOfs + 41 + 41 * idx, 40);
+	return mid(start + startOfs + 41 + 41 * idx, 40);
 }
 
 const QStringList Rev::parents() const {
@@ -1168,8 +1141,8 @@ const QStringList Rev::parents() const {
 	if (parentsCnt == 0)
 		return QStringList();
 
-	int ofs = start + boundaryOfs + 41;
-	return QStringList::split(" ", mid(ofs, 41 * parentsCnt - 1));
+	int ofs = start + startOfs + 41;
+	return QStringList::split(' ', mid(ofs, 41 * parentsCnt - 1));
 }
 
 int Rev::indexData() { // fast path here, less then 4% of load time
@@ -1188,10 +1161,13 @@ int Rev::indexData() { // fast path here, less then 4% of load time
 	- a terminating '\0'
 */
 	int last = ba.size() - 1;
-	boundaryOfs = uint(ba[start] == '-');
+
+	// take in account --boundary and --left-right options
+	startOfs = uint(ba.at(start) == '-' || ba.at(start) == '<' || ba.at(start) == '>');
+	boundary = startOfs && ba.at(start) == '-';
 
 	parentsCnt = 0;
-	int idx = start + boundaryOfs + 40;
+	int idx = start + startOfs + 40;
 	while (idx < last && ba[idx] == ' ') {
 		idx += 41;
 		parentsCnt++;
