@@ -22,7 +22,7 @@ ListView::ListView(Domain* dm, Git* g, QListView* l, FileHistory* f, const QFont
                    QObject(dm), d(dm), git(g), lv(l), fh(f) {
 
 	st = &(d->st);
-	filterNextContextMenuRequest = false;
+	filterNextContextMenuRequest = currentChangedEmitted = false;
 	setupListView(fnt);
 	clear(); // to init some stuff
 
@@ -228,8 +228,9 @@ void ListView::on_newRevsAdded(const FileHistory* f, const QValueVector<QString>
 
 void ListView::on_currentChanged(QListViewItem* item) {
 
+	currentChangedEmitted = true;
 	SCRef selRev(item ? (static_cast<ListViewItem*>(item))->sha() : "");
-	if (st->sha() != selRev) { // to avoid looping
+	if (st->sha() != selRev) { // to avoid looping and spurious calls
 		st->setSha(selRev);
 		st->setSelectItem(true);
 		UPDATE_DOMAIN(d);
@@ -240,6 +241,17 @@ void ListView::on_mouseButtonPressed(int b, QListViewItem* item, const QPoint&,
 
 	if (item && b == Qt::LeftButton)
 		d->setReadyToDrag(true);
+
+	/*
+	   when current item is not selected, as example after opening a new
+	   file history tab, then clicking on it does not fires currentChanged()
+	   signal. We use this kludge to trigger that case and force an update.
+	   It works because this slot is called always after on_currentChanged()
+	*/
+	if (item && !currentChangedEmitted && item == lv->currentItem() && item->isSelected())
+		on_currentChanged(item);
+
+	currentChangedEmitted = false; // reset the flag
 }
 
 void ListView::on_clicked(QListViewItem*) {
