@@ -20,6 +20,7 @@ MyProcess::MyProcess(QObject *go, Git* g, const QString& wd, bool err) : QProces
 	git = g;
 	workDir = wd;
 	receiver = NULL;
+	runOutput = NULL;
 	errorReportingEnabled = err;
 	canceling = async = false;
 	exitStatus = true;
@@ -37,14 +38,14 @@ bool MyProcess::runAsync(SCRef rc, QObject* rcv, SCRef buf) {
 	return true;
 }
 
-bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
+bool MyProcess::runSync(SCRef rc, QByteArray* ro, QObject* rcv, SCRef buf) {
 
 	async = false;
 	runCmd = rc;
 	runOutput = ro;
 	receiver = rcv;
-	if (runOutput != NULL)
-		*runOutput = "";
+	if (runOutput)
+		runOutput->resize(0);
 
 	setupSignals();
 	if (!launchMe(runCmd, buf))
@@ -70,7 +71,7 @@ bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
 
 	EM_AFTER_PROCESS_EVENTS;
 
-	if (git->curContext() != NULL)
+	if (git->curContext())
 		qDebug("ASSERT in MyProcess::runSync, context is %p "
 		       "instead of NULL", (void*)git->curContext());
 
@@ -84,10 +85,10 @@ void MyProcess::setupSignals() {
 	connect(git, SIGNAL(cancelAllProcesses()), this, SLOT(on_cancel()));
 	connect(this, SIGNAL(readyReadStdout()), this, SLOT(on_readyReadStdout()));
 	connect(this, SIGNAL(processExited()), this, SLOT(on_processExited()));
-	if (receiver != NULL) {
+	if (receiver) {
 		connect(this, SIGNAL(readyReadStderr()), this, SLOT(on_readyReadStderr()));
-		connect(this, SIGNAL(procDataReady(const QString&)),
-		        receiver, SLOT(on_procDataReady(const QString&)));
+		connect(this, SIGNAL(procDataReady(const QByteArray&)),
+		        receiver, SLOT(on_procDataReady(const QByteArray&)));
 		connect(this, SIGNAL(eof()), receiver, SLOT(on_eof()));
 	}
 	Domain* d = git->curContext();
@@ -126,29 +127,25 @@ void MyProcess::on_readyReadStdout() {
 // can became full and an hang occurs, so we read all data as soon
 // as possible and store it in runOutput
 
-	const QString tmp(readStdout());
 	if (canceling)
 		return;
 
-	if (receiver != NULL)
-		emit procDataReady(tmp);
+	if (receiver)
+		emit procDataReady(readStdout());
 
-	else if (runOutput != NULL)
-		runOutput->append(tmp);
+	else if (runOutput)
+		QGit::baAppend(*runOutput, readStdout());
 }
 
 void MyProcess::on_readyReadStderr() {
 
-	const QString tmp(readStderr());
 	if (canceling)
 		return;
 
-	if (receiver != NULL)
-		emit procDataReady(tmp); // redirect to stdout
-	else {
+	if (receiver)
+		emit procDataReady(readStderr()); // redirect to stdout
+	else
 		dbs("ASSERT in myReadFromStderr: NULL receiver");
-		return;
-	}
 }
 
 void MyProcess::on_processExited() {
@@ -175,17 +172,12 @@ void MyProcess::on_cancel() {
 	QProcess::tryTerminate();
 }
 
-void MyProcess::parseArgs(SCRef cmd, SList args) {
-
-	args = splitArgList(cmd);
-}
-
 const QStringList MyProcess::splitArgList(SCRef cmd) {
 // return argument list handling quotes and double quotes
 // substring, as example from:
-// cmd arg1 "some thing" arg2='some value'
-// to
-// sl = <cmd/arg1/some thing/arg2='some value'>
+// cmd some_arg "some thing" v='some value'
+// to (comma separated fields)
+// sl = <cmd,some_arg,some thing,v='some value'>
 
 	// early exit the common case
 	if (!(   cmd.contains(QGit::QUOTE_CHAR)
