@@ -18,12 +18,12 @@ class MyProcess : public QProcess {
 Q_OBJECT
 public:
 	MyProcess(QObject *go, Git* g, const QString& wd, bool reportErrors);
-	bool runSync(SCRef runCmd, QString* runOutput, QObject* rcv, SCRef buf);
+	bool runSync(SCRef runCmd, QByteArray* runOutput, QObject* rcv, SCRef buf);
 	bool runAsync(SCRef rc, QObject* rcv, SCRef buf);
-	static void parseArgs(SCRef cmd, SList args);
+	static const QStringList splitArgList(SCRef cmd);
 
 signals:
-	void procDataReady(const QString&);
+	void procDataReady(const QByteArray&);
 	void eof();
 
 public slots:
@@ -39,12 +39,11 @@ private:
 	bool launchMe(SCRef runCmd, SCRef buf);
 	void sendErrorMsg(bool notStarted = false);
 	static void restoreSpaces(QString& newCmd, SCRef sepChar);
-	static const QStringList splitArgList(SCRef cmd);
 
 	QObject* guiObject;
 	Git* git;
 	QString runCmd;
-	QString* runOutput;
+	QByteArray* runOutput;
 	QString workDir;
 	QObject* receiver;
 	bool errorReportingEnabled;
