@@ -11,7 +11,10 @@
  data in each file where QGit namespace is included.
 
 */
+#include <sys/types.h> // used by chmod()
+#include <sys/stat.h>  // used by chmod()
 #include <qsettings.h>
+#include <qdir.h>
 #include <qfile.h>
 #include "common.h"
 
@@ -110,8 +113,16 @@ void QGit::writeSetting(SCRef key, SCRef value, SCRef group) {
 }
 
 // misc helpers
-bool QGit::stripPartialParaghraps(SCRef src, QString* dst, QString* prev) {
+bool QGit::stripPartialParaghraps(const QByteArray& ba, QString* dst, QString* prev) {
 
+	QString src(ba);
+	// handle rare case of a '\0' inside content
+	while (src.length() < ba.size() && ba.at(src.length()) == '\0') {
+		int start = src.length() + 1;
+		QByteArray tail;
+		tail.duplicate(ba.data() + start, ba.size() - start);
+		src.append(" ").append(tail); // a space instead of '\0' so sizes match
+	}
 	int idx = src.findRev('\n');
 	if (idx == -1) {
 		prev->append(src);
@@ -123,9 +134,9 @@ bool QGit::stripPartialParaghraps(SCRef src, QString* dst, QString* prev) {
 	return true;
 }
 
-bool QGit::writeToFile(SCRef fileName, SCRef data) {
+bool QGit::writeToFile(SCRef fileName, SCRef data, bool setExecutable) {
 
-	QFile file(QFile::encodeName(fileName));
+	QFile file(fileName);
 	if (!file.open(IO_WriteOnly)) {
 		dbp("ERROR: unable to write file %1", fileName);
 		return false;
@@ -133,13 +144,36 @@ bool QGit::writeToFile(SCRef fileName, SCRef data) {
 	QTextStream stream(&file);
 	stream << data;
 	file.close();
+	if (setExecutable) {
+		if (chmod(fileName, 0755)) {
+			dbp("ERROR: unable to set executable bit to file %1", fileName);
+			QDir d;
+			d.remove(fileName);
+			return false;
+		}
+	}
+	return true;
+}
+
+bool QGit::writeToFile(SCRef fileName, const QByteArray& data, bool setExecutable) {
+
+	QFile file(fileName);
+	if (!file.open(IO_WriteOnly)) {
+		dbp("ERROR: unable to write file %1", fileName);
+		return false;
+	}
+	QDataStream stream(&file);
+	stream.writeRawBytes(data.data(), data.size());
+	file.close();
+	if (setExecutable)
+		chmod(fileName, 0755);
 	return true;
 }
 
 bool QGit::readFromFile(SCRef fileName, QString& data) {
 
 	data = "";
-	QFile file(QFile::encodeName(fileName));
+	QFile file(fileName);
 	if (!file.open(IO_ReadOnly)) {
 		dbp("ERROR: unable to read file %1", fileName);
 		return false;
@@ -149,3 +183,24 @@ bool QGit::readFromFile(SCRef fileName, QString& data) {
 	file.close();
 	return true;
 }
+
+void QGit::baAppend(QByteArray** baPtr, const char* ascii, int len) {
+
+	QByteArray* ba = *baPtr;
+	uint oldSize = 0;
+	if (ba) {
+		oldSize = ba->size();
+		ba->resize(oldSize + len, QGArray::SpeedOptim);
+	} else {
+		ba = new QByteArray(len);
+		*baPtr = ba;
+	}
+	// no detach here, no copy on write
+	memcpy(ba->data() + oldSize, ascii, len);
+}
+
+void QGit::baAppend(QByteArray& ba, const QByteArray& src) {
+
+	QByteArray* baPtr = &ba;
+	baAppend(&baPtr, src.data(),  src.size());
+}
