@@ -12,18 +12,28 @@
 #include <qsettings.h>
 #include <qregexp.h>
 #include "common.h"
+#include "git.h"
 #include "rangeselectimpl.h"
 
 using namespace QGit;
 
 RangeSelectImpl::RangeSelectImpl(QWidget* par, QString* r, const QStringList& tl,
-                                 bool* q) : RangeSelectBase(par, 0, true) {
+                                 bool* q, Git* g) : RangeSelectBase(par, 0, true) {
 	range = r;
 	quit = q;
+	git = g;
 	*quit = true; // if user press ESC everything works as expected
 
 	QStringList otl;
 	orderTags(tl, otl);
+
+	// check if top tag is current HEAD
+	if (!otl.empty()) {
+		const QString tagSha(git->getRefSha(otl.first(), Git::TAG, false));
+		if (git->checkRef(tagSha, Git::CUR_BRANCH))
+			// in this case remove from list to avoid an empty view
+			otl.pop_front();
+	}
 	comboBoxTo->insertStringList(otl);
 	comboBoxTo->insertItem("HEAD", 0);
 	comboBoxFrom->insertStringList(otl);
@@ -43,7 +53,7 @@ void RangeSelectImpl::orderTags(const QStringList& src, QStringList& dst) {
 	// match a (dotted) number + something else + a number + EOL
 	QRegExp re("[\\d\\.]+([^\\d\\.]+\\d+$)");
 
-	// in ASCII the space ' ' (32) cames before '!' (33) and both
+	// in ASCII the space ' ' (32) comes before '!' (33) and both
 	// before the rest, we need this to correctly order a sequence like
 	//
 	//	[v1.5, v1.5-rc1, v1.5.1] --> [v1.5.1, v1.5, v1.5-rc1]
@@ -52,8 +62,8 @@ void RangeSelectImpl::orderTags(const QStringList& src, QStringList& dst) {
 	const QString noRcMark("!$$%%"); // an impossible to find string starting with a '!'
 
 	QStringList tmpLs;
-	loopList(it1, src) {
-		const QString& s = *it1;
+	FOREACH_SL (it, src) {
+		const QString& s = *it;
 		if (re.search(s) != -1) {
 			QString t(s);
 			tmpLs.append(t.insert(re.pos(1), rcMark));
@@ -62,8 +72,8 @@ void RangeSelectImpl::orderTags(const QStringList& src, QStringList& dst) {
 	}
 	tmpLs.sort();
 	dst.clear();
-	loopList(it2, tmpLs) {
-		const QString& s = *it2;
+	FOREACH_SL (it, tmpLs) {
+		const QString& s = *it;
 		if (s.endsWith(noRcMark))
 			dst.prepend(s.left(s.length() - noRcMark.length()));
 		else {
