@@ -9,10 +9,12 @@
 
 #include "rangeselectbase.h"
 
+class Git;
+
 class RangeSelectImpl: public RangeSelectBase {
 Q_OBJECT
 public:
-	RangeSelectImpl(QWidget* par, QString* range, const QStringList& tl, bool* quit);
+	RangeSelectImpl(QWidget* par, QString* range, const QStringList& tl, bool* quit, Git* g);
 
 public slots:
 	void pushButtonOk_clicked();
@@ -26,6 +28,7 @@ private:
 
 	QString* range;
 	bool* quit;
+	Git* git;
 	QString fromBckUp;
 	QString toBckUp;
 };
