@@ -21,7 +21,7 @@ class RevDesc: public QTextBrowser {
 Q_OBJECT
 public:
 	RevDesc(QWidget* parent, const char* name);
-	void setDomain(Domain* dm) { d = dm; };
+	void setDomain(Domain* dm) { d = dm; }
 
 protected:
 	virtual QPopupMenu* createPopupMenu(const QPoint& pos);
