@@ -22,6 +22,7 @@
 #include <qdragobject.h>
 #include <qtoolbutton.h>
 #include <qtabwidget.h>
+#include <qaction.h>
 #include "common.h"
 #include "git.h"
 #include "domain.h"
@@ -122,9 +123,8 @@ void RevsView::viewPatch(bool newTab) {
 
 		connect(linkedPatchView->tab()->listBoxFiles, SIGNAL(doubleClicked(QListBoxItem*)),
 			m(), SLOT(on_fileList_doubleClicked(QListBoxItem*)));
-
-		connect(m(), SIGNAL(updateRevDesc()), pv, SLOT(on_updateRevDesc()));
 	}
+	connect(m(), SIGNAL(updateRevDesc()), pv, SLOT(on_updateRevDesc()));
 	connect(m(), SIGNAL(closeAllTabs()), pv, SLOT(on_closeAllTabs()));
 	pv->st = st;
 	UPDATE_DM_MASTER(pv, false);
@@ -149,7 +149,8 @@ void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
 
 void RevsView::on_updateRevDesc() {
 
-	SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE));
+	bool showHeader = m()->ActShowDescHeader->isOn();
+	SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE, showHeader));
 	tab()->textBrowserDesc->setText(d);
 	tab()->textBrowserDesc->setCursorPosition(0, 0);
 }
@@ -209,8 +210,10 @@ bool RevsView::doUpdate(bool force) {
 		// This could happen as example after removing a tree filter.
 		// At least populate secondary panes
 		st.setSha(((ListViewItem*)item)->sha());
-		st.setSelectItem(false);
-		customEvent(new UpdateDomainEvent(false)); // will be queued immediately
+		//st.setSelectItem(false); TODO remove selectItem from state properties
+		UpdateDomainEvent* e = new UpdateDomainEvent(false);
+		customEvent(e); // will be queued immediately
+		delete e;
 	}
 	return (found || st.sha().isEmpty());
 }
@@ -274,8 +277,8 @@ void RevsView::on_droppedRevisions(SCList remoteRevs) {
 		m()->statusBar()->message(tmp);
 		return;
 	}
-	bool commit, fold;
-	if (!m()->askApplyPatchParameters(&commit, &fold))
+	bool workDirOnly, fold;
+	if (!m()->askApplyPatchParameters(&workDirOnly, &fold))
 		return;
 
 	// ok, let's go
@@ -310,7 +313,7 @@ void RevsView::on_droppedRevisions(SCList remoteRevs) {
 			break;
 		}
 		SCRef fn(dr.absFilePath(dr[0]));
-		if (!git->applyPatchFile(fn, commit, fold, Git::optDragDrop))
+		if (!git->applyPatchFile(fn, fold, Git::optDragDrop))
 			break;
 
 		dr.remove(fn);
@@ -322,7 +325,7 @@ void RevsView::on_droppedRevisions(SCList remoteRevs) {
 	else
 		m()->statusBar()->message("Failed to import revision " + QString::number(revNum--));
 
-	if (!commit && (revNum > 0))
+	if (workDirOnly && (revNum > 0))
 		git->resetCommits(revNum);
 
 	dr.rmdir(dr.absPath()); // 'dr' must be already empty
