@@ -9,25 +9,16 @@
 
 #include <qguardedptr.h>
 #include "revbase.h" // needed by moc_* file to understand tab() function
-#include "exceptionmanager.h"
 #include "common.h"
 #include "domain.h"
 
+class MainImpl;
 class Git;
-class QListBoxItem;
-class QPopupMenu;
-class QTextEdit;
-class QTextBrowser;
-class QListBox;
-class QListViewItem;
+class FileHistory;
 class ListView;
-class ListViewItem;
-class RevDesc;
 class ListBoxFiles;
 class PatchView;
 class TreeView;
-class MainImpl;
-class FileHistory;
 
 class RevsView : public Domain {
 Q_OBJECT
@@ -42,10 +33,11 @@ public:
 private slots:
 	void on_loadCompleted(const FileHistory*, const QString& stats);
 	void on_lanesContextMenuRequested(const QStringList&, const QStringList&);
-	void on_droppedRevisions(const QString&, const QStringList&, const QString&);
+	void on_droppedRevisions(const QStringList&);
+	void on_updateRevDesc();
 
 protected:
-	virtual bool doUpdate();
+	virtual bool doUpdate(bool force);
 
 private:
 	friend class MainImpl;
