@@ -31,15 +31,18 @@ protected slots:
 	void lineEditTemplate_textChanged(const QString& s);
 	void lineEditCommitExtraOptions_textChanged(const QString& s);
 	void comboBoxCodecs_activated(int i);
+	void comboBoxUserSrc_activated(int i);
 	void pushButtonExtDiff_clicked();
 	void pushButtonFont_clicked();
 
 private:
 	void setupCodecList(QStringList& list);
 	void setupCodecsCombo();
+	void userInfo();
 
 	Git* git;
 	static const char* en[];
+	QStringList _uInfo;
 };
 
 #endif
