@@ -106,7 +106,7 @@ void DirItem::setOpen(bool b) {
 // ******************************* TreeView ****************************
 
 TreeView::TreeView(Domain* dm, Git* g, QListView* lv) : QObject(dm),
-                   d(dm), git(g), listView(lv) {
+                   d(dm), git(g), listView(lv), treeIsValid(false) {
 
 	st = &(d->st);
 	ignoreCurrentChanged = false;
@@ -135,6 +135,16 @@ void TreeView::initMimePix() {
 	setMimePix("perl",           m->lblPl->pixmap());
 	setMimePix("pl",             m->lblPl->pixmap());
 	setMimePix("py",             m->lblPy->pixmap());
+	setMimePix("java",           m->lblJava->pixmap());
+	setMimePix("jar",            m->lblJava->pixmap());
+	setMimePix("tar",            m->lblTar->pixmap());
+	setMimePix("gz",             m->lblTar->pixmap());
+	setMimePix("tgz",            m->lblTar->pixmap());
+	setMimePix("zip",            m->lblTar->pixmap());
+	setMimePix("bz",             m->lblTar->pixmap());
+	setMimePix("bz2",            m->lblTar->pixmap());
+	setMimePix("html",           m->lblHtml->pixmap());
+	setMimePix("xml",            m->lblHtml->pixmap());
 
 	m->lblFolder->hide();
 	m->lblFolderOpen->hide();
@@ -146,6 +156,9 @@ void TreeView::initMimePix() {
 	m->lblSh->hide();
 	m->lblPl->hide();
 	m->lblPy->hide();
+	m->lblJava->hide();
+	m->lblTar->hide();
+	m->lblHtml->hide();
 }
 
 void TreeView::on_currentChanged(QListViewItem* item) {
@@ -313,14 +326,13 @@ void TreeView::update() {
 			item = item->itemBelow();
 		}
 	}
+	// check if st->fileName() has been deleted by a patch older than this tree
 	if (item && treeIsValid) {
 		listView->clearSelection();
 		listView->setSelected(item, true);
 		listView->setCurrentItem(item); // calls on_currentChanged()
 		listView->ensureItemVisible(item);
-	} else
-		; // st->fileName() has been deleted by a patch older than this tree
-
+	}
 	listView->setUpdatesEnabled(true);
 	listView->triggerUpdate();
 	restoreStuff();
